=begin
Copyright 2005-3009, TIG
Permission to use, copy, modify, and distribute this software for 
any purpose and without fee is hereby granted, provided the above
copyright notice appear in all copies.
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------
Name        : Archiver
Description : A tool to Archive Models
Menu Item   : File -> Archiver/Open_Archives/Archive_Option
Context Menu: None
Author      : TIG
Usage       :
  Pick 'Archiver' to make an archive copy of the current model.
  It saves the model (in its current state) into a sub-folder 
  within the model's folder called 'Archives'.  It's named with 
  a [suffix] - date (yy-mm-dd_hh-mm-ss), 
  e.g. Model[09-12-18_12-34-56].skp.  
  The separators can be customized from '-' to '_' etc, 
  just change one line - sep="-" to sep="_" etc...
  If later on you don't save the main model with those changes 
  then that archive copy still has those changes current at the 
  time of the archiving.
  
  If there is an 'Archives' folder then an extra 'File' menu  
  item is added - "Open Archives"...
  
  'Archive Option' is an additional function that saves the model as it 
  stands - say 'Model.skp' - then it makes an archive of it - say 
  'Model[09-04-16_12-25-05].skp' and then opens a 'save_as' dialog in 
  which it will have preset the next available unused Option name - say 
  'Model[Option-01].skp' - which you are prompted to save_as: you can 
  choose to overwrite an earlier option if desired - if so you are 
  warned about overwriting it.  The Options are kept in the same folder 
  as the original.  It is recommended that you do not change the naming 
  convention...  The model is then saved.  The current SketchUp window 
  then shows this latest Option.
  Options can be archived in the same way as other models - in the 
  example above the archive's name might then be 
  'Model[Option-01][09-04-16_12-30-55].skp'...
  If you subsequently use 'Archive Option' on an 'Option' itself it then 
  names the new Option with the next available free Option name - in 
  this case it would 'Model[Option-02].skp' and the process repeats...
  It uses the [-] characters as separators so don't change the Option's 
  name if you want it to be recognised as one by 'Archive Option'.
Date        : 12/2005
Type        : Tool
Versions    :
  1.1	18/12/05	First issue.
  1.2	27/12/05	Typo corrected in dialog path. 
			Username added to archive name.
	1.3	27/12/05pm	Unsaved model error trapped.
			Menu typo corrected.
	1.4	27/12/05pm+	Suffix options added you (sep="-"). 
			Moved to 'File>Archiver' menu, 
			also 'File>Open Archives' menu added.
	1.5	28/12/05	Suffix separator options changed.
			Suffix 01 v. 1 etc corrected.
		  Model Name removed from 'Open Archives' menu item.
			Open Recent FilePath bug fixed.
  1.6 06/04/09  Glitch fixed on model diectory NOT being set...
  1.6x          Ruby Console output added, for Mac glitch tester.
  1.7 07/04/09  User suffix option removed. Now fully Mac/PC compliant.
      'Open Archives' now selects last skp in folder as default.
  1.8 15/04/09  'Archive Option' added.
  1.9 16/04/09  'Archive Option' now saves original model first,
      tmp files duting processing now have 'time' as their name.
#------------------------------------------------------------------------
=end
require 'sketchup.rb'
###
class Archiver
TOO_BIG=1024 * 1024 * 2 # >2Mb ???
###---------------------------------------
def Archiver::make()
  model=Sketchup.active_model
  mpath=model.path
 if mpath==""
  UI.messagebox("You can't Archive a New Model until you have Saved it !   ")
  return nil
 end#if
mpath=mpath.tr("\\","/")
paths=mpath.split("/")
paths=paths-[paths.last]
path=paths.join("/")+"/"
title=model.title
###[7, 41, 21, 18, 12, 2005, 0, 352, false, "GMT Standard Time"]
t=Time::now.to_a
sep="-" ### change to sep="_", or =" ", ="~" etc as desired...
sep_="_" ### change to sep_="-", or =" ", ="~" etc as desired...
### put ### in front of sfix type - or _ that's NOT required..
yy=t[5].to_s[2..4]
mm=(100+t[4]).to_s[1..3]### so "01" not "1"etc
dd=(100+t[3]).to_s[1..3]
hh=(100+t[2]).to_s[1..3]
mn=(100+t[1]).to_s[1..3]
ss=(100+t[0]).to_s[1..3]
sfix=yy+sep+mm+sep+dd+sep_+hh+sep+mn+sep+ss
name=title+"["+sfix+"].skp"
archive=path+"Archives"
Dir.mkdir(archive) if not FileTest.directory?(archive)
archname=archive+"/"+name
Dir.chdir(path)### ensure saves into model directory
tmp=Time.now.to_i.to_s+".tmp"
model.save(tmp)
### check if in Model folder and process it
 if FileTest.file?(path+tmp)
   Archiver.copy((path+tmp),archname)
   File.delete(path+tmp)
   ###remove ### below to get dialog when made...
   ###UI.messagebox("   Archive:\n\n#{name}\n\n   made in:\n\n#{archive}\\   \n\n")
   #return nil
 end#if
end#def
# copy files
 def Archiver::catname(from,to)
    if FileTest.directory?(to)
      to +
	    if to =~ /\\/
	      if to[-1,1] != '\\' then '\\' end + basename(from)
	    else
	      if to[-1,1] != '/' then '/' end + basename(from)
	    end
    else
      to
    end
 end#def
###
 def Archiver::syscopy(from,to)
    to=Archiver.catname(from,to)
    fsize=File.size(from)
    fsize=1024 if fsize < 512
    fsize=TOO_BIG if fsize > TOO_BIG
    fmode=File.stat(from).mode
    tpath=to
    from=File.open(from,"r")
    from.binmode
    to=File.open(to,"w")
    to.binmode
    begin
      while true
	      r=from.sysread(fsize)
	      rsize=r.size
        w=0
	        while w < rsize
	          t=to.syswrite(r[w,rsize - w])
	          w += t
	        end
      end
     rescue EOFError
      ret=true
     rescue
      ret=false
     ensure
      to.close
      from.close
    end
    File.chmod(fmode,tpath)
    ret
end#def
###
def Archiver::copy(from,to)#,verbose=false)
    #$stderr.print from," --> ",catname(from,to),"\n" if verbose
    Archiver.syscopy(from,to)
 end#def
###
 def Archiver::archives()
   model=Sketchup.active_model
   mpath=model.path
   if mpath==""
      UI.messagebox("NO 'Archives' Folder - Un-saved New Model !   ")
      return nil
   end#if
   mpath=mpath.tr("\\","/")
   paths=mpath.split("/")
   paths=paths-[paths.last]
   path=paths.join("/")+"/"
   archive=path+"Archives/"
   ### fix for PC openpanel - sep on PC = \\, but Mac = /
   archive=archive.tr("/","\\") if [PLATFORM].grep(/mswin/)==[PLATFORM]
   title=model.title
  if not FileTest.directory?(archive)
      UI.messagebox("NO 'Archives' Folder for '#{title}' !  ")      
      return nil
   end#if
   skp=(Dir.entries(archive).sort!).last
   arch=UI.openpanel("Open Archive File (#{title})...",archive,skp)
   Sketchup.open_file(arch) if arch
 end#def
###
 def Archiver::option()
  model=Sketchup.active_model
  mpath=model.path
  mpatho=mpath
  if mpath==""
    UI.messagebox("You can't Archive/Option a New Model until you have Saved it !   ")
    return nil
  end#if
  mpath=mpath.tr("\\","/")
  paths=mpath.split("/")
  paths=paths-[paths.last]
  path=paths.join("/")+"/"
  title=model.title
  mpath=path.tr("/","\\")# fix for PC v Mac ?
### 1st archive
  Archiver.make()
### 2nd save model as is with tmp name
  tmp=Time.now.to_i.to_s+".tmp"
  model.save(tmp)
### 3rd save_as ti/di/fi
### test if already [Option-]
  otherfiles=Dir.entries(mpath)
  num=0; numi=nil
  otherfiles.each{|file|
    if file.include?("[")and numi=file.split("[")[1].split("]")[0].split("-")[1]
      num=numi.to_i if numi.to_i > num
    end#if
  }### num is highest vale option
  opt="[Option-"  
  numi=0
  while numi <= num
    numi=numi+1
  end#if
  title=title.split("[")[0]
  num=numi.to_s
  num = "0"+num if numi <10 #leading zero
  suf=opt+num+"]"
  title2=title+suf+".skp"
  save=UI.savepanel("Save the Model '"+title+"' as...",mpath,title2)
  if save
    savef=save.split("\\").last
    savef=savef.split("/").last
    model.save(savef)
    Sketchup.open_file(save)
    ### move original back now closed
    File.rename(tmp,title+".skp")
  else ### erase tmp file
    File.delete(tmp)
  end#if
 end#def
###
 def Archiver::file_menu()
   @file_menu=UI.menu("File").add_item("Open Archives"){Archiver.archives}
 end#def
###
end#class
###
#--------- menu -----------------------------
if(not file_loaded?("Archiver.rb"))
   add_separator_to_menu("File")
   UI.menu("File").add_item("Archiver"){Archiver.make}
   Archiver.file_menu
   UI.menu("File").add_item("Archive Option"){Archiver.option}
end#if
file_loaded("Archiver.rb")
#-------------------------------------------
